@@ -1,6 +1,4 @@
-"""
-Test deserialization routines that read from file
-"""
+"""Test deserialization routines that read from file"""
 import unittest
 import param
 import sys
@@ -12,7 +10,7 @@ from shutil import rmtree
 try:
     import numpy as np
     ndarray = np.array([[1,2,3],[4,5,6]])
-except:
+except ModuleNotFoundError:
     np = ndarray = None
 
 try:
@@ -20,19 +18,19 @@ try:
     pd_ver = pd.__version__.split('.')
     df = pd.DataFrame({'A':[1,2,3], 'B':[1.1,2.2,3.3]})
     modern_pd = pd if (int(pd_ver[0]) >= 1 and int(pd_ver[1]) >= 2) else None
-except:
+except ModuleNotFoundError:
     pd = df1 = df2 = modern_pd = None
 
 # The writer could be xlsxwriter, but the sufficient condition is the presence of
 # openpyxl
 try:
     import openpyxl as xlsxm
-except:
+except ModuleNotFoundError:
     xlsxm = None
 
 try:
     import odf as ods
-except:
+except ModuleNotFoundError:
     ods = None
 
 # prior to pandas version 1.2, xlrd was always the default excel reader (though it
@@ -42,27 +40,27 @@ try:
     import xlrd as xls
     if int(xls.__version__.split('.')[0]) > 2:
         raise Exception()
-except:
+except ModuleNotFoundError:
     if modern_pd is None:
         xlsxm = None
 
 try:
     import pyarrow.feather as feather
-except:
+except ModuleNotFoundError:
     feather = None
 
 try:
     import fastparquet as parquet
-except:
+except ModuleNotFoundError:
     parquet = None
 try:
     import pyarrow as parquet
-except:
+except ModuleNotFoundError:
     pass
 
 try:
     import tables as hdf5
-except:
+except ModuleNotFoundError:
     hdf5 = None
 
 np_skip = skipIf(np is None, "NumPy is not available")
@@ -206,8 +204,8 @@ class TestFileDeserialization(unittest.TestCase):
     @hdf5_skip
     def test_data_frame_hdf5(self):
         path = '{}/val.h5'.format(self.temp_dir)
-        P.data_frame.to_hdf(path, 'df')
+        P.data_frame.to_hdf(path, key='df')
         self._test_deserialize_array(P, path, 'data_frame')
         path = '{}/val.hdf5'.format(self.temp_dir)
-        P.data_frame.to_hdf(path, 'df')
+        P.data_frame.to_hdf(path, key='df')
         self._test_deserialize_array(P, path, 'data_frame')
