@@ -1,6 +1,4 @@
-"""
-Unit test for Parameterized.
-"""
+"""Unit test for Parameterized."""
 import inspect
 import re
 import unittest
@@ -17,7 +15,6 @@ import pytest
 import random
 
 from param import parameterized, Parameter
-from param._utils import _dict_update
 from param.parameterized import (
     ParamOverrides,
     Undefined,
@@ -271,15 +268,19 @@ class TestParameterized(unittest.TestCase):
         assert C.name == 'AA'
 
     def test_constant_parameter_modify_class_before(self):
-        """Test you can set on class and the new default is picked up
-        by new instances"""
+        """
+        Test you can set on class and the new default is picked up
+        by new instances
+        """
         TestPO.const=9
         testpo = TestPO()
         self.assertEqual(testpo.const,9)
 
     def test_constant_parameter_modify_class_after_init(self):
-        """Test that setting the value on the class doesn't update the instance value
-        even when the instance value hasn't yet been set"""
+        """
+        Test that setting the value on the class doesn't update the instance value
+        even when the instance value hasn't yet been set
+        """
         oobj = []
         class P(param.Parameterized):
             x = param.Parameter(default=oobj, constant=True)
@@ -333,7 +334,6 @@ class TestParameterized(unittest.TestCase):
 
     def test_basic_instantiation(self):
         """Check that instantiated parameters are copied into objects."""
-
         testpo = TestPO()
 
         self.assertEqual(testpo.inst,TestPO.inst)
@@ -523,7 +523,6 @@ class TestParameterized(unittest.TestCase):
 
     def test_values(self):
         """Basic tests of params() method."""
-
         # CB: test not so good because it requires changes if params
         # of PO are changed
         assert 'name' in param.Parameterized.param.values()
@@ -588,6 +587,26 @@ class TestParameterized(unittest.TestCase):
             else:
                 assert obj is TestPO.param[p]
 
+    def test_param_error_unsafe_ops_before_initialized(self):
+        class P(param.Parameterized):
+
+            x = param.Parameter()
+
+            def __init__(self, **params):
+                with pytest.raises(
+                    RuntimeError,
+                    match=re.escape(
+                        'Looking up instance Parameter objects (`.param.objects()`) until '
+                        'the Parameterized instance has been fully initialized is not allowed. '
+                        'Ensure you have called `super().__init__(**params)` in your Parameterized '
+                        'constructor before trying to access instance Parameter objects, or '
+                        'looking up the class Parameter objects with `.param.objects(instance=False)` '
+                        'may be enough for your use case.',
+                    )
+                ):
+                    self.param.objects()
+        P()
+
     def test_instance_param_getitem(self):
         test = TestPO()
         assert test.param['inst'] is not TestPO.param['inst']
@@ -833,6 +852,26 @@ class TestParameterized(unittest.TestCase):
         assert t.inst == 'foo'
         assert t.notinst == 1
 
+    def test_constant_readonly_parameterized(self):
+        class ParamClass(param.Parameterized):
+            x = param.Number()
+
+        pc1 = ParamClass(name="FOO1")
+        pc2 = ParamClass(name="FOO2")
+
+        class P(param.Parameterized):
+            ro = param.Parameter(pc1, constant=True)
+            const = param.Parameter(pc2, readonly=True)
+            ro_i = param.Parameter(pc1, constant=True, instantiate=True)
+            const_i = param.Parameter(pc2, readonly=True, instantiate=True)
+
+        p = P()
+
+        assert p.ro.name == 'FOO1'
+        assert p.const.name == 'FOO2'
+        assert p.ro_i.name.startswith('ParamClass0')
+        assert p.const_i.name == 'FOO2'
+
 
 class some_fn(param.ParameterizedFunction):
     __test__ = False
@@ -1337,7 +1376,7 @@ class TestShallowCopyMutableAttributes:
 
             __slots__ = ['container']
 
-            _slot_defaults = _dict_update(Parameter._slot_defaults, container=None)
+            _slot_defaults = dict(Parameter._slot_defaults, container=None)
 
             def __init__(self, default=Undefined, *, container=Undefined, **kwargs):
                 super().__init__(default=default, **kwargs)
@@ -1480,7 +1519,6 @@ def test_namespace_class():
     assert _dir(P) == [
         '_param__parameters',
         '_param__private',
-        '_param_watchers',
         'foo',
         'name',
         'param',
@@ -1502,7 +1540,6 @@ def test_namespace_inst():
     assert _dir(p) == [
         '_param__parameters',
         '_param__private',
-        '_param_watchers',
         'foo',
         'name',
         'param',
@@ -1656,3 +1693,108 @@ def test_parameterized_signature_subclass_multiple_inheritance_init_different():
     check_signature(A, ['a1', 'a2', 'name'])
     assert B.__signature__ is None
     assert C.__signature__ is None
+
+
+def test_inheritance_with_incompatible_defaults():
+    class A(param.Parameterized):
+        p = param.String()
+
+    class B(A): pass
+
+    with pytest.raises(
+        RuntimeError,
+        match=re.escape(
+            "Number parameter 'C.p' failed to validate its "
+            "default value on class creation. "
+            "The Parameter type changed between class 'C' "
+            "and one of its parent classes (B, A) which made it invalid. "
+            "Please fix the Parameter type."
+            "\nValidation failed with:\nNumber parameter 'C.p' only takes numeric values, not <class 'str'>."
+        )
+    ):
+        class C(B):
+            p = param.Number()
+
+
+def test_inheritance_default_validation_with_more_specific_type():
+    class A(param.Parameterized):
+        p = param.Tuple(default=('a', 'b'))
+
+    class B(A): pass
+
+    with pytest.raises(
+        RuntimeError,
+        match=re.escape(
+            "NumericTuple parameter 'C.p' failed to validate its "
+            "default value on class creation. "
+            "The Parameter type changed between class 'C' "
+            "and one of its parent classes (B, A) which made it invalid. "
+            "Please fix the Parameter type."
+            "\nValidation failed with:\nNumericTuple parameter 'C.p' only takes numeric values, not <class 'str'>."
+        )
+    ):
+        class C(B):
+            p = param.NumericTuple()
+
+
+def test_inheritance_with_changing_bounds():
+    class A(param.Parameterized):
+        p = param.Number(default=5)
+
+    class B(A): pass
+
+    with pytest.raises(
+        RuntimeError,
+        match=re.escape(
+            "Number parameter 'C.p' failed to validate its "
+            "default value on class creation. "
+            "The Parameter is defined with attributes "
+            "which when combined with attributes inherited from its parent "
+            "classes (B, A) make it invalid. Please fix the Parameter attributes."
+            "\nValidation failed with:\nNumber parameter 'C.p' must be at most 3, not 5."
+        )
+    ):
+        class C(B):
+            p = param.Number(bounds=(-1, 3))
+
+
+def test_inheritance_with_changing_default():
+    class A(param.Parameterized):
+        p = param.Number(default=5, bounds=(3, 10))
+
+    class B(A): pass
+
+    with pytest.raises(
+        RuntimeError,
+        match=re.escape(
+            "Number parameter 'C.p' failed to validate its "
+            "default value on class creation. "
+            "The Parameter is defined with attributes "
+            "which when combined with attributes inherited from its parent "
+            "classes (B, A) make it invalid. Please fix the Parameter attributes."
+            "\nValidation failed with:\nNumber parameter 'C.p' must be at least 3, not 1."
+        )
+    ):
+        class C(B):
+            p = param.Number(default=1)
+
+
+def test_inheritance_with_changing_class_():
+    class A(param.Parameterized):
+        p = param.ClassSelector(class_=int, default=5)
+
+    class B(A): pass
+
+    with pytest.raises(
+        RuntimeError,
+        match=re.escape(
+            "ClassSelector parameter 'C.p' failed to validate its "
+            "default value on class creation. "
+            "The Parameter is defined with attributes "
+            "which when combined with attributes inherited from its parent "
+            "classes (B, A) make it invalid. Please fix the Parameter attributes."
+            "\nValidation failed with:\nClassSelector parameter 'C.p' value must be an instance of str, not 5."
+        )
+    ):
+        class C(B):
+            p = param.ClassSelector(class_=str)
