@@ -5,7 +5,7 @@ import time
 import param
 import pytest
 
-from param.parameterized import Skip
+from param.parameterized import Skip, resolve_ref
 from param.reactive import bind, rx
 
 class Parameters(param.Parameterized):
@@ -136,6 +136,20 @@ def test_nested_list_parameter_ref():
     p2.string_list = [p3.param.string, 'another']
     assert p2.string_list == ['foo', 'another']
 
+def test_nested_nested_parameter_ref_not_overwritten():
+    # Ensure refs are not removed when updating another ref
+    p = Parameters(string='bar')
+    p2 = Parameters(string='foo')
+    p3 = Parameters(string=p.param.string, string_list=[p2.param.string, 'other'])
+    p4 = Parameters()
+
+    assert p3.string_list == ['foo', 'other']
+    p3.string = p4.param.string
+    p.string = 'fizz'
+    p2.string = 'buzz'
+    assert p3.string == 'string'
+    assert p3.string_list == ['buzz', 'other']
+
 def test_nested_dict_key_parameter_ref():
     p = Parameters()
     p2 = Parameters(dictionary={p.param.string: 'value'})
@@ -286,3 +300,50 @@ async def test_generator_ref_cancelled():
     assert task1.done()
     assert not task2.done()
     assert len(threads) == 2
+
+def test_resolve_ref_parameter():
+    p = Parameters()
+    refs = resolve_ref(p.param.string)
+    assert len(refs) == 1
+    assert refs[0] is p.param.string
+
+def test_resolve_ref_depends_method():
+    p = Parameters()
+    refs = resolve_ref(p.formatted_string)
+    assert len(refs) == 1
+    assert refs[0] is p.param.string
+
+def test_resolve_ref_recursive_list():
+    p = Parameters()
+    nested = [[p.param.string]]
+    refs = resolve_ref(nested, recursive=True)
+    assert len(refs) == 1
+    assert refs[0] is p.param.string
+
+def test_resolve_ref_recursive_set():
+    p = Parameters()
+    nested = {(p.param.string,)}  # Parameters aren't hashable
+    refs = resolve_ref(nested, recursive=True)
+    assert len(refs) == 1
+    assert refs[0] is p.param.string
+
+def test_resolve_ref_recursive_tuple():
+    p = Parameters()
+    nested = ((p.param.string,),)
+    refs = resolve_ref(nested, recursive=True)
+    assert len(refs) == 1
+    assert refs[0] is p.param.string
+
+def test_resolve_ref_recursive_dict():
+    p = Parameters()
+    nested = {'0': {'0': p.param.string}}
+    refs = resolve_ref(nested, recursive=True)
+    assert len(refs) == 1
+    assert refs[0] is p.param.string
+
+def test_resolve_ref_recursive_slice():
+    p = Parameters()
+    nested = [slice(p.param.string)]
+    refs = resolve_ref(nested, recursive=True)
+    assert len(refs) == 1
+    assert refs[0] is p.param.string
