@@ -1,13 +1,12 @@
-"""
-Unit test for watch mechanism
-"""
+"""Unit test for watch mechanism"""
 import copy
+import re
 import unittest
 
 import param
 import pytest
 
-from param.parameterized import discard_events
+from param.parameterized import Skip, discard_events
 
 from .utils import MockLoggingHandler, warnings_as_excepts
 
@@ -112,6 +111,19 @@ class TestWatch(unittest.TestCase):
         obj.a = 2
         self.assertEqual(self.accumulator, 3)
 
+    def test_triggered_ignore_skip(self):
+        def accumulator(change):
+            if change.new > 1:
+                raise Skip()
+            self.accumulator += 1
+
+        obj = SimpleWatchExample()
+        obj.param.watch(accumulator, 'a')
+        obj.a = 1
+        self.assertEqual(self.accumulator, 1)
+        obj.a = 2
+        self.assertEqual(self.accumulator, 1)
+
     def test_discard_events_decorator(self):
         def accumulator(change):
             self.accumulator += change.new
@@ -639,37 +651,6 @@ class TestWatch(unittest.TestCase):
 
         obj.param.watch(lambda: '', ['a', 'b'])
 
-        with pytest.warns(FutureWarning):
-            pw = obj._param_watchers
-        assert isinstance(pw, dict)
-        for pname in ('a', 'b'):
-            assert pname in pw
-            assert 'value' in pw[pname]
-            assert isinstance(pw[pname]['value'], list) and len(pw[pname]['value']) == 1
-            assert isinstance(pw[pname]['value'][0], param.parameterized.Watcher)
-
-    def test_watch_watchers_modified(self):
-        accumulator = Accumulator()
-        obj = SimpleWatchExample()
-
-        obj.param.watch(accumulator, ['a', 'b'])
-
-        with pytest.warns(FutureWarning):
-            pw = obj._param_watchers
-        del pw['a']
-
-        obj.param.update(a=1, b=1)
-
-        assert accumulator.call_count() == 1
-        args = accumulator.args_for_call(0)
-        assert len(args) == 1
-        assert args[0].name == 'b'
-
-    def test_watch_watchers_exposed_public(self):
-        obj = SimpleWatchExample()
-
-        obj.param.watch(lambda: '', ['a', 'b'])
-
         pw = obj.param.watchers
         assert isinstance(pw, dict)
         for pname in ('a', 'b'):
@@ -678,7 +659,7 @@ class TestWatch(unittest.TestCase):
             assert isinstance(pw[pname]['value'], list) and len(pw[pname]['value']) == 1
             assert isinstance(pw[pname]['value'][0], param.parameterized.Watcher)
 
-    def test_watch_watchers_modified_public(self):
+    def test_watch_watchers_modified(self):
         accumulator = Accumulator()
         obj = SimpleWatchExample()
 
@@ -720,6 +701,25 @@ class TestWatch(unittest.TestCase):
         ):
             SimpleWatchExample.param.watchers = {}
 
+    def test_watch_error_unsafe_before_initialized(self):
+        class P(param.Parameterized):
+
+            x = param.Parameter()
+
+            def __init__(self, **params):
+                with pytest.raises(
+                    RuntimeError,
+                    match=re.escape(
+                        '(Un)registering a watcher on a partially initialized Parameterized instance '
+                        'is not allowed. Ensure you have called super().__init__(**) in the '
+                        'Parameterized instance constructor before trying to set up a watcher.',
+                    )
+                ):
+                    self.param.watch(print, 'x')
+
+        P()
+
+
 
 class TestWatchMethod(unittest.TestCase):
 
@@ -1042,3 +1042,21 @@ class TestTrigger(unittest.TestCase):
 
         example.picker.value += 1
         assert example.da == 3
+
+    def test_trigger_error_unsafe_before_initialized(self):
+        class P(param.Parameterized):
+
+            x = param.Parameter()
+
+            def __init__(self, **params):
+                with pytest.raises(
+                    RuntimeError,
+                    match=re.escape(
+                        'Triggering watchers on a partially initialized Parameterized instance '
+                        'is not allowed. Ensure you have called super().__init__(**params) in '
+                        'the Parameterized instance constructor before trying to set up a watcher.',
+                    )
+                ):
+                    self.param.trigger('x')
+
+        P()
